@@ -18,6 +18,7 @@ def test_chemparse():
                 'Mn(SO4)2(H2O)7':  {'H': 14.0, 'S': 2.0, 'Mn': 1, 'O': 15.0},
                 'Mg(SO4)2': {'Mg': 1, 'S': 2, 'O':8},
                 'Mg0.5Fe0.5' : {'Mg': 0.5, 'Fe': 0.5},
+                'Mg.3Fe.7' : {'Mg': 0.3, 'Fe': 0.7},
                 'CO': {'C': 1, 'O': 1} }
     for formula, cert in examples.items():
         ret = chemparse(formula)
@@ -47,6 +48,14 @@ def test_get_materials():
         else:
             assert (name not in known_materials)
 
+def test_get_filtered_materials():
+    sol = get_materials(categories=['solvent'])
+    assert 'water' in sol
+    assert 'acetone' in sol
+    assert 'gold'  not in sol
+    assert 'methane'  not in sol
+
+
 def test_material_mu1():
     en = np.linspace(8500, 9500, 21)
     known_mu = np.array([236.2, 232.4, 228.7, 225.1, 221.5, 218.1, 214.7,
