@@ -14,10 +14,14 @@ from xraydb import (chemparse, material_mu, material_mu_components,
                     xray_edge, xray_lines, xray_line, fluor_yield,
                     ck_probability, core_width, guess_edge,
                     xray_delta_beta, darwin_width, mirror_reflectivity,
+                    multilayer_reflectivity, coated_reflectivity,
                     ionchamber_fluxes, XrayDB)
 
 
-from xraydb.xray import chantler_data
+from xraydb.xray import (chantler_data, formula_to_mass_fracs,
+                         dynamical_theta_offset,
+                         _validate_mass_fracs, mass_fracs_to_molar_fracs,
+                         transmission_sample)
 
 def test_atomic_data():
     assert atomic_number('zn') == 30
@@ -171,6 +175,10 @@ def test_mu_elam():
     with pytest.raises(ValueError):
         mu_elam('Pb', en, kind='all')
 
+    # Check that array in -> array out, and scalar in -> scalar out
+    assert mu_elam('Pb', [1000]).shape == (1,)
+    assert isinstance(mu_elam('Pb', 1000), float)
+
 def test_elam_coh_incoh():
     en = np.linspace(13000, 17000, 51)
 
@@ -572,7 +580,55 @@ def test_mirror_reflectivity():
 
     assert_allclose(mirror_reflectivity('Pt', 0.001, 80000), 0.74, rtol=0.005)
 
+
+def test_multilayer_reflectivity():
+    # generated with xrt
+    # at low reflectivity, xrt and xraydb can differ up to 10%
+    xrt_r = np.array([0.9682573639725451, 0.9692894183725779,
+                      0.9551412674578308, 0.8718946307435569, 0.8843826288196437,
+                      0.8896591645964965, 0.8988360437159508, 0.9051158550840065,
+                      0.9104682228466688, 0.9143461645742539, 0.9171157874862768,
+                      0.9194212398362299, 0.9212651629926719, 0.9226454408036748,
+                      0.9236019502977926, 0.9241776792239949, 0.9244201997527346,
+                      0.9243126499599257, 0.9236380508617884, 0.9227686240812235,
+                      0.9216928483264080, 0.9199374247672095, 0.9175654141148161,
+                      0.9145854804063109, 0.9078206179439292])
+
+    stackup = ['Si', 'W']
+    thickness = [27, 18]
+    substrate = 'Si'
+    energy = np.linspace(1000, 10000, 25)
+    theta = 0.004
+    n_periods = 40
+    r = multilayer_reflectivity(stackup, thickness, substrate, theta, energy, n_periods)
+
+    assert_allclose(r, xrt_r, rtol=0.005)
+
+def test_coated_reflectivity():
+    # generated with xrt
+    # at low reflectivity, xrt and xraydb can differ up to 10%
+    xrt_r = np.array([0.955832355448275, 0.9607498872134996, 0.9633490690665248,
+                      0.9644769321246166, 0.9645676240437896, 0.9600938273346961,
+                      0.8509155394149021, 0.8638043361169211, 0.8786334026702274,
+                      0.8873502801010402, 0.8933107988079941, 0.8979355347158112,
+                      0.9016093183389469, 0.9047690354498116, 0.907622430698658,
+                      0.9101553863828132, 0.91235727241249, 0.9142224085825185, 0.915522720229593,
+                      0.9166717314973307, 0.9177369597483173, 0.91847146708513,
+                      0.9192559822224173, 0.9201650933145156, 0.920513174385813])
+
+    E = np.linspace(1000, 10000, 25)
+    theta = 4e-3
+    coating = 'Rh'
+    coating_thick = 300
+    substrate = 'Si'
+    r = coated_reflectivity(coating, coating_thick, substrate, theta, E)
+
+    assert_allclose(r, xrt_r, rtol=0.005)
+
+
+
 def test_darwin_width():
+
     def dw_dide(energy, crystal, hkl):
         """estimate darwin width in energy from min/max derivative"""
         dat = darwin_width(energy, crystal=crystal, hkl=hkl)
@@ -585,42 +641,123 @@ def test_darwin_width():
         return darwin_width(energy, crystal=crystal, hkl=hkl).energy_fwhm
 
     # fwhm from derivatives:
-    assert_allclose(dw_dide( 5000, 'Si', (1, 1, 1)), 0.685, rtol=0.01)
-    assert_allclose(dw_dide(10000, 'Si', (1, 1, 1)), 1.340, rtol=0.01)
+    assert_allclose(dw_dide( 5000, 'Si', (1, 1, 1)), 0.685, rtol=0.02)
+    assert_allclose(dw_dide(10000, 'Si', (1, 1, 1)), 1.340, rtol=0.02)
 
-    assert_allclose(dw_dide(10000, 'Si', (2, 2, 0)), 0.595, rtol=0.01)
-    assert_allclose(dw_dide(15000, 'Si', (2, 2, 0)), 0.891, rtol=0.01)
+    assert_allclose(dw_dide(10000, 'Si', (2, 2, 0)), 0.595, rtol=0.02)
+    assert_allclose(dw_dide(15000, 'Si', (2, 2, 0)), 0.891, rtol=0.02)
 
-    assert_allclose(dw_dide(10000, 'Si', (3, 1, 1)), 0.288, rtol=0.01)
-    assert_allclose(dw_dide(20000, 'Si', (3, 1, 1)), 0.565, rtol=0.01)
+    assert_allclose(dw_dide(10000, 'Si', (3, 1, 1)), 0.288, rtol=0.02)
+    assert_allclose(dw_dide(20000, 'Si', (3, 1, 1)), 0.565, rtol=0.02)
 
     # fwhm as cacluated
-    assert_allclose(dw_fwhm( 5000, 'Si', (1, 1, 1)), 0.723, rtol=0.01)
-    assert_allclose(dw_fwhm(10000, 'Si', (1, 1, 1)), 1.418, rtol=0.01)
+    assert_allclose(dw_fwhm( 5000, 'Si', (1, 1, 1)), 0.723, rtol=0.02)
+    assert_allclose(dw_fwhm(10000, 'Si', (1, 1, 1)), 1.418, rtol=0.02)
 
-    assert_allclose(dw_fwhm(10000, 'Si', (2, 2, 0)), 0.624, rtol=0.01)
-    assert_allclose(dw_fwhm(15000, 'Si', (2, 2, 0)), 0.927, rtol=0.01)
+    assert_allclose(dw_fwhm(10000, 'Si', (2, 2, 0)), 0.624, rtol=0.02)
+    assert_allclose(dw_fwhm(15000, 'Si', (2, 2, 0)), 0.927, rtol=0.02)
 
-    assert_allclose(dw_fwhm(10000, 'Si', (3, 1, 1)), 0.301, rtol=0.01)
-    assert_allclose(dw_fwhm(20000, 'Si', (3, 1, 1)), 0.593, rtol=0.01)
+    assert_allclose(dw_fwhm(10000, 'Si', (3, 1, 1)), 0.301, rtol=0.02)
+    assert_allclose(dw_fwhm(20000, 'Si', (3, 1, 1)), 0.593, rtol=0.02)
+
+def test_dynamical_offset():
+    t_off = dynamical_theta_offset(10000, 'Si', hkl=(1,1,1))*1e6
+
+    assert t_off > 24
+    assert t_off < 26
 
 
 def test_ionchamber_fluxes():
-        ic1 = ionchamber_fluxes(gas='helium', volts=1.25, length=200.0,
-                              energy=10000.0, sensitivity=1.e-9)
+    ic1 = ionchamber_fluxes(gas='helium', volts=1.25, length=20.0,
+                            energy=15000.0, sensitivity=1.e-9)
 
-        assert_allclose(ic1.photo, 16110895.3, rtol=0.01)
-        assert_allclose(ic1.transmitted, 15316549138.8, rtol=0.01)
+    ic2 = ionchamber_fluxes(gas='nitrogen', volts=1.25, length=20.0,
+                            energy=15000.0, sensitivity=1.e-6)
 
-        ic2 = ionchamber_fluxes(gas='nitrogen', volts=1.25, length=200.0,
-                                energy=10000.0, sensitivity=1.e-9)
+    ic3 = ionchamber_fluxes(gas={'nitrogen':0.5, 'helium': 0.5}, volts=1.25,
+                            length=20.0, energy=15000.0, sensitivity=1.e-6)
 
-        assert_allclose(ic2.photo, 13575282.2, rtol=0.01)
-        assert_allclose(ic2.incident, 23102328.0, rtol=0.01)
+    ic4 = ionchamber_fluxes(gas='Ar', volts=1.25, length=20.0,
+                            energy=15000.0, sensitivity=1.e-6)
 
-        ic3 = ionchamber_fluxes(gas={'nitrogen':0.5, 'helium': 0.5}, volts=1.25,
-                                length=200.0, energy=10000.0, sensitivity=1.e-9)
+    assert_allclose(ic1.transmitted/ic1.incident, 0.999, rtol=0.03)
+    assert_allclose(ic1.incident, 2.503e11, rtol=0.03)
 
-        assert_allclose(ic3.photo, 14843088.8, rtol=0.01)
-        assert_allclose(ic3.incident, 7737855176.4, rtol=0.01)
-        assert_allclose(ic3.transmitted, 7662654298.8, rtol=0.01)
+    assert_allclose(ic2.transmitted/ic2.incident, 0.970, rtol=0.03)
+    assert_allclose(ic2.incident, 3.75e11, rtol=0.03)
+
+    assert_allclose(ic3.transmitted/ic3.incident, 0.984, rtol=0.03)
+    assert_allclose(ic3.incident, 8.40e11, rtol=0.03)
+
+    assert_allclose(ic4.transmitted/ic4.incident, 0.493, rtol=0.03)
+    assert_allclose(ic4.incident, 1.394e10, rtol=0.03)
+
+
+
+def test_formula_to_mass_fracs():
+    mf1 = formula_to_mass_fracs('Fe2O3')
+    desired1 = {'Fe': 0.69943, 'O': 0.30056}
+    assert_allclose(mf1['Fe'], desired1['Fe'], rtol=0.001)
+    assert_allclose(mf1['O'], desired1['O'], rtol=0.001)
+    assert_allclose(sum(mf1.values()), 1.0, rtol=0.001)
+
+    mf2 = formula_to_mass_fracs({'Fe2O3':1, 'FeO':2})
+    desired2 = {'Fe': 0.7363, 'O': 0.2637}
+    assert_allclose(mf2['Fe'], desired2['Fe'], rtol=0.001)
+    assert_allclose(mf2['O'], desired2['O'], rtol=0.001)
+    assert_allclose(sum(mf2.values()), 1.0, rtol=0.001)
+
+
+def test_validate_mass_fracs():
+    mf = _validate_mass_fracs({'Fe2O3': 0.75, 'FeO': 0.25})
+    desired = {'Fe': 0.7189, 'O': 0.2811}
+    assert_allclose(mf['Fe'], desired['Fe'], rtol=0.001)
+    assert_allclose(mf['O'], desired['O'], rtol=0.001)
+    assert_allclose(sum(mf.values()), 1.0, rtol=0.001)
+
+
+def test_mass_fracs_to_molar_fracs():
+    """Circular test for consistency formula->mass fracs
+    and vice versa."""
+    formula = {'Fe':1, 'FeO': 3}
+    mass_fracs = formula_to_mass_fracs(formula)
+    molar_fracs = mass_fracs_to_molar_fracs(mass_fracs)
+    assert_allclose(molar_fracs['Fe'], 4 / 7, rtol=0.001)
+    assert_allclose(molar_fracs['O'], 3 / 7, rtol=0.001)
+
+
+def test_transmission_sample():
+    # Validate against Hephaestus result, see PR #16
+    result1 = transmission_sample(
+                    sample='Fe2O3',
+                    energy=7162,
+                    absorp_total=1,
+                    area=1,
+                    density=5.24
+                )
+    assert_allclose(result1.mass_total_mg, 3.506, rtol=0.001)
+    assert_allclose(result1.absorption_length_um, 6.7, rtol=0.01)
+
+    # Validate against Hephaestus result, see PR #16
+    result2 = transmission_sample(
+                    sample='Cu',
+                    energy=9029,
+                    absorp_total=1,
+                    area=1,
+                    density=8.94
+                )
+    assert_allclose(result2.mass_total_mg, 3.640, rtol=0.001)
+    assert_allclose(result2.absorption_length_um, 4.1, rtol=0.01)
+
+    # Validate against XAFSmass result, see PR #16
+    result3 = transmission_sample(
+                    sample={'Fe': 0.05, 'SiO2': -1},
+                    energy=7162,
+                    absorp_total=2.6,
+                    area=1.33,
+                    density=2.65
+                )
+    assert_allclose(result3.mass_components_mg['Fe'], 2.58, rtol=0.03)
+    assert_allclose(result3.absorbance_steps['Fe'], 0.706, rtol=0.06)
+    assert_allclose(result3.mass_total_mg, 51.7, rtol=0.02)
+    assert_allclose(result3.thickness_mm, 0.1466, rtol=0.02)
